@@ -3,6 +3,7 @@ import os
 import sys
 import csv
 import json
+import shlex
 # Pour affichage de dict
 import pprint
 # Pour décomprésser
@@ -37,15 +38,14 @@ class FastEval:
         #self.wcolor = bg('orange_1') + fg('white')
         self.wcolor = fg('orange_1')
         #self.icolor = bg('deep_sky_blue_2') + fg('white')
-        self.icolor = fg('deep_sky_blue_2')
+        #self.icolor = fg('medium_turquoise') + attr('bold')
+        self.icolor = fg('light_sea_green') + attr('bold')
         self.rcolor = attr('reset')
-
-        if args.ws:
-            self.workspace_path = os.path.abspath(os.path.expanduser(args.ws))
-            #self.workspace_path = os.path.expanduser(args.ws)
+        if args.workspace:
+            self.workspace_path = os.path.abspath(os.path.expanduser(args.workspace))
         else:
             self.workspace_path = os.path.join(os.getcwd(), 'submissions')
-        print(f'Using  {self.info_str(self.workspace_path)} as workspace.')
+        print(f'Using  {self.info_str(self.workspace_path)} as workspace. {self.info_str("✓")}')
 
         self.archive_path = os.path.expanduser(args.archive_path)
         if not os.path.exists(self.archive_path):
@@ -54,12 +54,13 @@ class FastEval:
                   file=sys.stderr)
             sys.exit()
 
+        self.verbosity = args.verbosity
         config_path = os.path.expanduser(args.config)
         assert os.path.isfile(config_path), "{} is not a file.".format(self.erro_str(config_path))
 
         with open(config_path, 'r') as fp:
             config = json.load(fp)
-        print('Loaded ' + self.info_str(config_path) + ' savefile.')
+        print(f'Loaded {self.info_str(config_path)} savefile. {self.info_str("✓")}')
         self.required_files = config['required_files']
 
         if len(config['reference_folder']) > 0:
@@ -69,13 +70,23 @@ class FastEval:
                   ' does not exist, exiting...'.format(self.erro_str(self.ref_path)),
                   file=sys.stderr)
                 sys.exit()
-            print('Using  {} as reference folder'.format(self.info_str(self.ref_path)))
+            print(f'Using  {self.info_str(self.ref_path)} as reference folder. {self.info_str("✓")}')
         else:
             self.ref_path = None
             print('Not using ref folder')
 
-        self.comp_cmd = config['compilation_commands']
-        self.exec_cmd = config['execution_commands']
+        if 'compilation_commands' in config:
+            self.comp_cmd = config['compilation_commands']
+        else:
+            self.comp_cmd = []
+        if 'execution_commands' in config:
+            self.exec_cmd = config['execution_commands']
+        else:
+            self.exec_cmd = []
+        if 'cleanup' in config:
+            self.cleanup_cmd = config['cleanup']
+        else:
+            self.cleanup_cmd = []
 
         self.submissions = {}
         # Chargement de la config
@@ -96,12 +107,16 @@ class FastEval:
         else:
             print('Processing {} projects...\n'.format(len(self.submissions)))
             self.check_prep()
-        #pprint.pprint(self.submissions)
-        self.exte_step(self.comp_cmd, step='1_comp', label='Compiling')
-        #pprint.pprint(self.submissions)
-        self.exte_step(self.exec_cmd, step='2_exec', label='Executing')
-        #pprint.pprint(self.submissions)
+
+        self.print_step_errors('0_prep')
         self.write_data()
+        self.exte_step(self.comp_cmd, step='1_comp', label='Compiling')
+        self.print_step_errors('1_comp')
+        self.write_data()
+        #self.exte_step(self.exec_cmd, step='2_exec', label='Executing')
+        self.print_step_errors('2_exec')
+        self.write_data()
+        self.export()
 
     def load_data(self):
         data_file = os.path.join(self.workspace_path, 'data.json')
@@ -113,9 +128,9 @@ class FastEval:
     
             self.pass_count = data['pass_count'] + 1
             self.submissions = data['submissions']
-            print('Loaded ' + self.info_str(data_file) + ' savefile.\n')
+            print(f'Loaded {self.info_str(data_file)} savefile. {self.info_str("✓")}\n')
         except FileNotFoundError:
-            print('Using  ' + self.info_str(data_file) + ' savefile.\n')
+            print(f'Using  {self.info_str(data_file)} savefile. {self.info_str("✓")}\n')
             self.pass_count = 0
     def write_data(self):
         data_file = os.path.join(self.workspace_path, 'data.json')
@@ -124,7 +139,7 @@ class FastEval:
                 json.dump({'pass_count': self.pass_count,
                            'submissions': self.submissions},
                           fp, sort_keys=True, indent=4, ensure_ascii=False)
-            print('Wrote  ' + self.info_str(data_file) + ' savefile.')
+            print(f'Wrote  {self.info_str(data_file)} savefile. {self.info_str("✓")}')
         except:
             print('Error while writing : \n => {}\n'.format(data_file),
                   file=sys.stderr)
@@ -175,7 +190,6 @@ class FastEval:
                 student_code = search_files(f, raw_dir)
                 # Filter files in a "__MACOS" directory
                 student_code = [s for s in student_code if '__MACOS' not in s]
-                print(student_code)
                 if len(student_code) == 1:
                     shutil.copyfile(student_code[0], os.path.join(eval_dir, f))
                 elif len(student_code) == 0:
@@ -195,7 +209,10 @@ class FastEval:
                 self.submissions[sub]['step'] = '1_comp'
     
         to_prep = [sub for sub in self.submissions if self.submissions[sub]['step'] == '0_prep']
-        print('           ' + self.erro_str('{} fails.'.format(len(to_prep))) + '\n')
+        if len(to_prep) == 0:
+            print(f'           0 fails. {self.info_str("✓")}')
+        else:
+            print('           ' + self.erro_str('{} fails.'.format(len(to_prep))) + '\n')
     def check_prep(self):
         to_check = [sub for sub in self.submissions if self.submissions[sub]['step'] == '0_prep']
         print('Checking   {} projects...'.format(len(to_check)))
@@ -213,29 +230,119 @@ class FastEval:
     
         to_check = [sub for sub in self.submissions if self.submissions[sub]['step'] == '0_prep']
         print('           ' + self.erro_str('{} fails.'.format(len(to_check))) + '\n')
-    def exte_step(self, cmd, step='1_comp', label='Compiling'):
+    def exte_step(self, cmd, step='1_comp', label='Compiling', timeout=10):
         to_exec = [sub for sub in self.submissions if self.submissions[sub]['step'] == step]
         print('{}  {} projects...'.format(label, len(to_exec)))
         root_dir = os.getcwd()
         for sub in to_exec:
             os.chdir(os.path.join(self.submissions[sub]['path'], 'eval'))
             comp_ok = True
+            timeout_raised = False
             for c in cmd:
-                completed_process = subprocess.run([c], capture_output=True, text=True, shell=True)
-                if completed_process.returncode == 1:
+                try:
+                    completed_process = subprocess.run([c], capture_output=True, text=True, shell=True, timeout=timeout)
+                    if completed_process.returncode == 1:
+                        comp_ok=False
+                    cond = [len(completed_process.stderr) > 0, len(completed_process.stdout)]
+                    if any(cond) and c not in self.submissions[sub]['steps'][step]:
+                        self.submissions[sub]['steps'][step][c] = {}
+                    if cond[0]:
+                        self.submissions[sub]['steps'][step][c]['stderr'] = completed_process.stderr.split('\n')
+                    if cond[1]:
+                        out = completed_process.stdout.split('\n')
+                        if len(out) > 20:
+                            out = out[:10] + ['.'] + ['truncated by fast-eval'] + ['.'] + out[-10:]
+                        self.submissions[sub]['steps'][step][c]['stdout'] = out
+                except Exception as e:
                     comp_ok=False
-                cond = [len(completed_process.stderr) > 0, len(completed_process.stdout) > 0]
-                if any(cond) and c not in self.submissions[sub]['steps'][step]:
-                    self.submissions[sub]['steps'][step][c] = {}
-                if cond[0]:
-                    self.submissions[sub]['steps'][step][c]['stderr'] = completed_process.stderr.split('\n')
-                if cond[1]:
-                    self.submissions[sub]['steps'][step][c]['stdout'] = completed_process.stdout.split('\n')
+                    if type(e) is subprocess.TimeoutExpired:
+                        self.submissions[sub]['steps'][step][c] = 'timeout'
+    
             if comp_ok:
                 self.submissions[sub]['step'] = self.next_step(step)
         os.chdir(root_dir)
         to_exec = [sub for sub in self.submissions if self.submissions[sub]['step'] == step]
-        print('           ' + self.erro_str('{} fails.'.format(len(to_exec))) + '\n')
+        if len(to_exec) == 0:
+            print(f'           0 fails. {self.info_str("✓")}')
+        else:
+            print('           ' + self.erro_str('{} fails.'.format(len(to_exec))) + '\n')
+        self.cleanup()
+    
+    def cleanup(self):
+        for c in self.cleanup_cmd:
+            completed_process = subprocess.run(shlex.split(c))
+            if completed_process.returncode == 0:
+                print(f'Cleanup : {c} {self.info_str("✓")}')
+            else:
+                print(f'Cleanup : {c} {self.erro_str("❌")}')
+    def export(self):
+        outpath = os.path.join(self.workspace_path, 'readme.org')
+        with open(outpath, 'w') as f:
+            f.write("#+title: Rapport d'évaluation\n")
+            for s in self.submissions:
+              step = self.submissions[s]['step']
+              f.write(f'** {s}\n')
+    
+              # Section erreur prep
+              if step == '0_prep':
+                  f.write(f'*** Erreurs de préparation\n')
+                  for k, v in self.submissions[s]['steps']['0_prep'].items():
+                      f.write(f'{k} :\n')
+                      for i in v:
+                          f.write(f' - {i}\n')
+              # Section erreur comp
+              if step != '0_prep':
+                  ce = self.submissions[s]['steps']['1_comp']
+                  if ce:
+                      f.write(f'*** Erreurs de compilation\n')
+                  for k, v in ce.items():
+                      f.write(f'#+begin_src bash\n')
+                      f.write(f'{k}\n')
+                      f.write('#+end_src\n')
+                      f.write('\n#+name: stderror\n')
+                      f.write(f'#+begin_example\n')
+                      for line in v['stderr']:
+                          f.write(f'{line}\n')
+                      f.write('\n#+end_example\n')
+    
+              # Section avec code rendu
+              if step != '0_prep':
+                  f.write(f'*** code\n')
+                  for sf in self.required_files:
+                      f.write(f'**** {sf}\n')
+                      # Détermination du langage
+                      l = os.path.splitext(sf)[-1][1:]
+                      if l == 'py':
+                        l = python
+                      if l == 'sh':
+                        l = bash
+                      # Copie du code de l'étudiant
+                      f.write(f'#+begin_src {l}\n')
+                      with open(os.path.join(self.submissions[s]['path'], 'eval', sf), 'r') as cf:
+                            f.write(cf.read())
+                      f.write('\n#+end_src\n')
+    
+              # Section retour exécution
+              if step != '0_prep' and step != '1_comp':
+                  e = self.submissions[s]['steps']['2_exec']
+                  if e:
+                      f.write(f"*** Retours d'éxécution\n")
+                  for k, v in e.items():
+                      f.write(f'#+begin_src bash\n')
+                      f.write(f'{k}\n')
+                      f.write('#+end_src\n')
+                      if 'stderr' in v:
+                          f.write('\n#+name: stderror\n')
+                          f.write(f'#+begin_example\n')
+                          for line in v['stderr']:
+                              f.write(f'{line}\n')
+                          f.write('#+end_example\n')
+                      if 'stdout' in v:
+                          f.write('\n#+name: stdout\n')
+                          f.write(f'#+begin_example\n')
+                          for line in v['stdout']:
+                              f.write(f'{line}\n')
+                          f.write('#+end_example\n')
     
     def next_step(self, step):
         if step == '0_prep':
@@ -252,3 +359,11 @@ class FastEval:
         return self.wcolor + str(msg) + self.rcolor
     def info_str(self, msg):
         return self.icolor + str(msg) + self.rcolor
+    def print_step_errors(self, step):
+        to_print = [sub for sub in self.submissions if self.submissions[sub]['step'] == step]
+        if self.verbosity >= 1 and len(to_print) > 0:
+            print(f"Fail list : {to_print}\n")
+        if self.verbosity > 1:
+            for s in to_print:
+                print(f'{s}\'s errors : \n {self.submissions[s]["steps"][step]}')
+        print("\n")
